server: Convert com_addatt() to lopsub.
[paraslash.git] / signal.c
index aa63c8b5461a9627c021a514b83b7c8ee1067067..5d6e6e45dcd8e728c1bfcc6e08899bf08d4fc512 100644 (file)
--- a/signal.c
+++ b/signal.c
@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2004 Andre Noll <maan@tuebingen.mpg.de>
  *
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
@@ -7,10 +7,15 @@
 
 #include <signal.h>
 #include <sys/types.h>
+#include <regex.h>
 
 #include "para.h"
 #include "error.h"
 #include "fd.h"
+#include "list.h"
+#include "string.h"
+#include "sched.h"
+#include "signal.h"
 
 static int signal_pipe[2];
 
@@ -22,19 +27,21 @@ static int signal_pipe[2];
  * during the application's startup part, followed by subsequent calls
  * to para_install_sighandler() for each signal that should be caught.
  *
- * para_signal_init() installs a generic signal handler which is used for all
- * signals simultaneously. When a signal arrives, this generic signal handler
- * writes the corresponding signal number to the signal pipe so that the
- * application can test for pending signals simply by checking the signal pipe
- * for reading, e.g. by using the select(2) system call.
+ * A generic signal handler is used for all signals simultaneously. When a
+ * signal arrives, the signal handler writes the number of the signal received
+ * to one end of the signal pipe. The application can test for pending signals
+ * by checking if the file descriptor of the other end of the signal pipe is
+ * ready for reading, see select(2).
  *
* \return This function either succeeds or calls exit(2) to terminate
* the current process. On success, the file descriptor of the signal pipe is
* returned.
  */
-int para_signal_init(void)
+struct signal_task *signal_init_or_die(void)
 {
+       struct signal_task *st;
        int ret;
+
+       PARA_NOTICE_LOG("setting up signal handling\n");
        if (pipe(signal_pipe) < 0) {
                ret = -ERRNO_TO_PARA_ERROR(errno);
                goto err_out;
@@ -45,21 +52,30 @@ int para_signal_init(void)
        ret = mark_fd_nonblocking(signal_pipe[1]);
        if (ret < 0)
                goto err_out;
-       return signal_pipe[0];
+       st = para_calloc(sizeof(*st));
+       st->fd = signal_pipe[0];
+       return st;
 err_out:
        PARA_EMERG_LOG("%s\n", para_strerror(-ret));
        exit(EXIT_FAILURE);
 }
 
-/*
- * just write one integer to signal pipe
- */
+/* Write the signal number to signal pipe. */
 static void generic_signal_handler(int s)
 {
+       /*
+        * Signal handlers that make system calls must save a copy of errno on
+        * entry to the handler and restore it on exit, to prevent the
+        * possibility of overwriting a errno value that had previously been
+        * set in the main program.
+        */
+       int save_errno = errno;
        ssize_t ret = write(signal_pipe[1], &s, sizeof(int));
 
-       if (ret == sizeof(int))
+       if (ret == sizeof(int)) {
+               errno = save_errno;
                return;
+       }
        if (ret < 0)
                PARA_EMERG_LOG("%s\n", strerror(errno));
        else
@@ -213,9 +229,12 @@ int para_next_signal(fd_set *rfds)
 }
 
 /**
- * Close the write end of the signal pipe.
+ * Close the write end of the signal pipe, deallocate resources.
+ *
+ * \param st The pointer obtained earlier from signal_init_or_die().
  */
-void para_signal_shutdown(void)
+void signal_shutdown(struct signal_task *st)
 {
        close(signal_pipe[1]);
+       free(st);
 }
@@ -22,19 +27,21 @@ static int signal_pipe[2];
  *
- * \return This function either succeeds or calls exit(2) to terminate
- * the current process. On success, the file descriptor of the signal pipe is
- * returned.
+ * \return This function either succeeds or calls exit(2) to terminate the
+ * current process. On success, a signal task structure is returned.
  */